/*
 *  node-arduino: Control your Arduino with Node
 *
 *  Copyright (c) 2010 Tobias Schneider
 *  node-arduino is freely distributable under the terms of the MIT license.
 */

var sys = require('sys'),
	SerialPort = require('serialport').SerialPort,
	parsers = require('serialport').parsers;

var util = require("util")
  , events = require("events")
  , EventEmitter = events.EventEmitter;

const SERIAL_BAUDRATE = 115200;

const OPC_PIN_MODE         = 0x01;
const OPC_DIGITAL_READ     = 0x02;
const OPC_DIGITAL_WRITE    = 0x03;
const OPC_ANALOG_REFERENCE = 0x04;
const OPC_ANALOG_READ      = 0x05;
const OPC_ANALOG_WRITE     = 0x06;

exports.HIGH = 0x01;
exports.LOW  = 0x00;

exports.INPUT  = 0x00;
exports.OUTPUT = 0x01;

exports.true  = 0x01;
exports.false = 0x00;

exports.EXTERNAL = 0x00;
exports.DEFAULT  = 0x01;
exports.INTERNAL = 0x03;

Board = function (path) {
	events.EventEmitter.call(this);

	// Callback Function Map for read functions
	this.callback = {};
	this.sp = new SerialPort(path, {baudrate: SERIAL_BAUDRATE, parser: parsers.readline('\n')});

	var onData = (function(callback) {
			return function( data ) {
				var pin, value;
				data = +data;
				if ( data && data > 1 ) {
					pin = data >> 16;
					value = data & 0xFFFF;
	
					// There is no callback function.
					if ( !callback['pin'+pin] ) {
						sys.puts('no callback');
					} else {
						callback['pin'+pin](value);
					}
				}
			};	
		})(this.callback);

	this.sp.on("data", onData);

	this.sp.on("error",
			
		(function(_self){				
			return function(err) {
				console.log('[SerialPort] Connection Error : '+err);
				_self.emit('reconnect',err);
			}
		})(this)
	);

	this.on("error", function(err) {
		console.log('[ArduinoJS] Connection Error : '+err);	
	});

	this.on("reconnect", 
		(function(_self){				
			return function(err) {
				console.log('[ArduinoJS] Try Reconnecting...');
				_self.sp = new SerialPort(path, {baudrate: SERIAL_BAUDRATE, parser: parsers.readline('\n')});
		 	};
		})(this)
	);

    this.on("close",
		(function(_self){
			return function(err) {
                if ( !_self.sp ) {
                    return ;
                }
				_self.sp.close();
		 	};
		})(this)
	);
};

util.inherits(Board, events.EventEmitter);

Board.prototype.pinMode = function (pin, mode) {
	try {
		this.sp.write(new Buffer([OPC_PIN_MODE, pin, mode]), 3);
	} catch(e) {
		this.emit('error', e);
	}
};

Board.prototype.digitalRead = function (pin, callback) {
	try {
		this.sp.write(new Buffer([OPC_DIGITAL_READ, pin]), 2);

		if ( typeof callback == 'function' ) {
			this.callback['pin'+pin] = callback;
		}
	} catch(e) {
		console.log('digitalRead');
		this.emit('error', e);
	}
};

Board.prototype.digitalWrite = function (pin, val) {
	try {
		this.sp.write(new Buffer([OPC_DIGITAL_WRITE, pin, val]), 3);
	} catch(e) {
		console.log('digitalWrite');
		this.emit('error', e);
	}
};
			  
Board.prototype.analogReference = function (type) {
	try {
		this.sp.write(new Buffer([OPC_ANALOG_REFERENCE, type]), 2);
	} catch(e) {
		console.log('analogReference');
		this.emit('error', e);
	}
};

Board.prototype.analogRead = function (pin, callback) {
	try {
		this.sp.write(new Buffer([OPC_ANALOG_READ, pin]), 2);

		if ( typeof callback == 'function' ) {
			this.callback['pin'+pin] = callback;
		}
	} catch(e) {
		console.log('analogRead');
		this.emit('error', e);
	}
};

Board.prototype.analogWrite = function (pin, val) {
	try {
		this.sp.write(new Buffer([OPC_ANALOG_WRITE, pin, val]), 3);
	} catch(e) {
		console.log('analogWrite');
		this.emit('error', e);
	}
};

Board.prototype.close = function () {
	try {
		this.sp.close();
	} catch(e) {
		console.log('close');
		this.emit('error', e);
	}
};


exports.connect = function (path) {
	return new Board(path);
};
